#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   vis_mask.py
@Time    :   2022/05/06 16:34:46
@Author  :   Moyan 
'''
import os
import cv2
import numpy as np
import matplotlib.pyplot as plt

def show_mask(img, seg_map, palette=None, show=False, classes_num=256, opacity=0.5, out_file=None):
    def init_palette():
        state = np.random.get_state()
        np.random.seed(42)
        palette = np.random.randint(0, 255, size=(classes_num, 3))
        np.random.set_state
        return palette
    img = img.copy()
    img = img[..., ::-1]
    print(np.unique(seg_map))
    if palette is None: palette = init_palette()
    color_seg = np.zeros((seg_map.shape[0], seg_map.shape[1], 3), dtype=np.uint8)
    for label, color in enumerate(palette):
        # print(label, color)
        color_seg[seg_map==label, :] = color
    # convert to BGR
    color_seg = color_seg[..., ::-1]

    img = img * (1-opacity) + color_seg * opacity
    img = img.astype(np.uint8)

    plt.figure()
    plt.imshow(img)
    plt.axis('off')

    if show:
        plt.show()
    if out_file is not None:
        cv2.imwrite(out_file, img)

    plt.close()

# def main():
#     img_path = r"D:\Code\sunc\generate_fake_idcard\gen_xj_0_front.jpg"
#     mask_path = r"D:\Code\sunc\generate_fake_idcard\gen_xj_0_front.png"
#     im = cv2.imread(img_path)
#     mask = cv2.imread(mask_path, 0)

#     out_file = "vis.png"
#     show_mask(im, mask, show=True, out_file=out_file)
# if __name__ == '__main__':
#     main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   TFLite.py
@Time    :   2022/11/23 16:08:02
@Author  :   Moyan 
'''
import os
import numpy as np
import tensorflow as tf
from abc import ABCMeta, abstractclassmethod


class TFLiteModel(metaclass=ABCMeta):

    def __init__(self, model_path: str) -> None:
        assert os.path.exists(model_path), "model_path is not exists!"
        self.interpreter = tf.lite.Interpreter(model_path)
        self.interpreter.allocate_tensors()
        self._input_details = self.interpreter.get_input_details()
        self._output_details = self.interpreter.get_output_details()

    @abstractclassmethod
    def preprocess(self):
        pass

    @abstractclassmethod
    def postprocess(self):
        pass
    
    def __forward(self, input_tensor):
        assert len(self._input_details)==1, "only one input accept!"
        assert tuple(self._input_details[0]["shape"])==input_tensor.shape, "input_tensor shape is wrong!"

        self.interpreter.set_tensor(self._input_details[0]['index'],  input_tensor)
        self.interpreter.invoke()

        results = []
        for i in range(len(self._output_details)):
            output = self.interpreter.get_tensor(self._output_details[i]["index"])
            results.append(output)
        return results

    def run(self, x:np.ndarray):
        input_tensor = self.preprocess(x)
        output_tensor = self.__forward(input_tensor)
        return self.postprocess(output_tensor)

    def get_input_detais(self):
        return self._input_details
    
    def get_output_details(self):
        return self._output_details



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   ssd_tflite.py
@Time    :   2022/11/23 21:07:44
@Author  :   Moyan 
@ssd_model_path: https://github.com/ValYouW/tflite-win-c/blob/master/ssd_mobilenet_v3_float.tflite
'''
import cv2
import numpy as np
from .TFLite import TFLiteModel


class SSDTFliteModel(TFLiteModel):
    def __init__(self, 
        model_path: str, input_size=320, threshhold=0.5) -> None:
        super().__init__(model_path)
        self.origin_height=0
        self.origin_weight=0
        self.input_size=input_size
        self.threshhold=threshhold
        
    def preprocess(self, image:np.ndarray):
        self.origin_height, self.origin_weight = image.shape[:2]
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        im = cv2.resize(image, (self.input_size, self.input_size), interpolation=cv2.INTER_LINEAR)
        im = im.astype("float32")
        im = im*(1.0/127.5)-1.0
        im = im[None,:,:,:]
        return im 

    def postprocess(self, output_tensor):
        '''
        output_tensor: [[bbox], [class], [score], [num_output]]
            bbox:   (1, 10, 4)
            class:  (1, 10)
            score:  (1, 10)
            num_output: (1,)
        '''
        predict_bboxes = output_tensor[0].squeeze() # (10, 4)
        predict_classes = output_tensor[1].squeeze() # (10,)
        predict_scores = output_tensor[2].squeeze() # (10,)
        select_index = predict_scores>self.threshhold
        predict_bboxes = predict_bboxes[select_index]
        predict_classes = predict_classes[select_index]
        predict_scores = predict_scores[select_index]
        # rescale to origin size
        predict_bboxes[:, 0] *= self.origin_height
        predict_bboxes[:, 2] *= self.origin_height
        predict_bboxes[:, 1] *= self.origin_weight
        predict_bboxes[:, 3] *= self.origin_weight
        # y1, x1, y2, x2 -> x1, y1, x2, y2
        predict_bboxes[:, [0, 1, 2, 3]] = predict_bboxes[:, [1, 0, 3, 2]]
        # index 0 is background
        predict_classes = predict_classes.astype(np.int32) + 1
        assert len(predict_bboxes) == len(predict_classes) == len(predict_scores)
        return predict_bboxes, predict_classes, predict_scores


#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   tflite_detection.py
@Time    :   2022/11/23 16:34:58
@Author  :   Moyan 
'''
import sys
sys.path.insert(0, "D:\Code\AI-in-air\moyan\test")


import numpy as np
from abc import abstractmethod
from TFLite import TFLiteModel

class DetectModel(TFLiteModel):
    def __init__(self, model_path: str) -> None:
        super().__init__(model_path)
        pass
    
    @abstractmethod
    def preprocess(self):
        pass

    @abstractmethod
    def postprocess(self):
        pass
    
    def run(self, x:np.ndarray):
        input_tensor = self.preprocess(x)
        output_tensor = self.forward(input_tensor)
        return self.postprocess(output_tensor)




# -*- coding: utf-8 -*-
# Created on Mar-14-22 17:32
# @site: https://github.com/moyans
# @author: moyan
import os
import random
from PIL import Image
import numpy as np
from collections import namedtuple

cls_template = namedtuple('cls', ['name', 'id', 'color'])
# label_key_val_map = [
#     cls_template('bg', 0, (0, 0, 0)),
#     cls_template('cls1', 1, (255, 128, 128)),
#     cls_template('cls2', 2, (255, 255, 128)),
#     cls_template('cls3', 3, (128, 255, 128)),
#     cls_template('cls4', 4, (128, 255, 255)),
#     cls_template('cls5', 5, (0, 128, 255)),
#     cls_template('cls6', 6, (255, 0, 0)),
#     cls_template('cls7', 7, (0, 128, 192)),
#     cls_template('cls8', 8, (255, 0, 255)),
#     cls_template('cls9', 9, (255, 128, 64)),
#     cls_template('cls10', 10, (128, 64, 0)),
#     cls_template('cls11', 11, (128, 128, 0)),
# ]

label_key_val_map = [
    cls_template('bg', 0, (0, 0, 0)),
    cls_template('back_all', 9, (255, 128, 128)),
    cls_template('back_organize', 7, (255, 255, 128)),
    cls_template('back_effetive_data', 6, (128, 255, 128)),
    cls_template('front_all', 8, (128, 255, 255)),
    cls_template('front_name', 1, (0, 128, 255)),
    cls_template('front_gender', 2, (255, 0, 0)),
    cls_template('front_nation', 3, (0, 128, 192)),
    cls_template('front_addr', 4, (255, 0, 255)),
    cls_template('front_idnum', 5, (255, 128, 64)),
]


def get_putpalette(Clss, color_other=[0, 0, 0]):
    '''
    灰度图转8bit彩色图
    :param Clss:颜色映射表
    :param color_other:其余颜色设置
    :return:
    '''
    putpalette = []
    for cls in Clss:
        putpalette += list(cls.color)
    putpalette += color_other * (255 - len(Clss))
    return putpalette

def main():

    gray_mask_path = 'demo.png'
    gray_8bit_mask_path = 'demo_8bit.png'

    src = Image.open(gray_mask_path)
    mat = np.array(src, dtype=np.uint8)
    dst = Image.fromarray(mat, 'P')

    bin_colormap = get_putpalette(label_key_val_map)
    dst.putpalette(bin_colormap)
    dst.save(gray_8bit_mask_path)
    src.close()

def batch_dir():

    img_dir = r"D:\Code\sunc\generate_fake_idcard\res\xj\mask"
    save_dir = os.path.join(os.path.dirname(img_dir), "mask8bit")
    if not os.path.exists(save_dir): os.makedirs(save_dir)

    img_list = os.listdir(img_dir)
    for ind, names in enumerate(img_list):
        if ind%100 == 0: print(ind, names)
        name, suffix = os.path.splitext(names)
        if suffix != ".png":continue
        gray_mask_path = os.path.join(img_dir, name+".png")
        mask8bit_path = os.path.join(save_dir, name+".png")
        # mask = cv2.imread(mask_path)
        # new_mask = encode_segmap(mask)
        # cv2.imwrite(mask_path, new_mask)

        src = Image.open(gray_mask_path)
        mat = np.array(src, dtype=np.uint8)
        dst = Image.fromarray(mat, 'P')

        bin_colormap = get_putpalette(label_key_val_map)
        dst.putpalette(bin_colormap)
        dst.save(mask8bit_path)
        src.close()

    
if __name__ == '__main__':
    # main()
    batch_dir()


# coding: utf-8
from __future__ import absolute_import
from __future__ import print_function

from .tools import vis_segmentation
from .labelme_json_to_png import labelme_json_to_png
from .vis_mask import show_mask